/*
HappyDays - A Birthday displayer for the PalmPilot
Copyright (C) 1999-2001 JaeMok Jeong

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _DATEBOOK_H__
#define _DATEBOOK_H__

#include <PalmOS.h>
#include "section.h"

typedef struct {
    UInt16	renamedCategories;	// bitfield of categories with a different
                                // name
    char 	categoryLabels[dmRecNumCategories][dmCategoryLength];
    UInt8	categoryUniqIDs[dmRecNumCategories];
    UInt8	lastUniqID;	// Uniq IDs generated by the device are between
    // 0 - 127.  Those from the PC are 128 - 255.
    UInt8	reserved1;	// from the compiler word aligning things
    UInt16	reserved2;
    UInt8	startOfWeek;
} ApptAppInfoType;


typedef ApptAppInfoType * ApptAppInfoPtr;


/************************************************************
 *
 * Appointment Database constants.
 *
 *************************************************************/
#define apptMaxPerDay 		100		// max appointments displayable on a day.
#define apptNoTime			-1		// start time of an untimed appt.
#define apptNoEndDate		-1		// end date of appts that repeat forever
#define apptNoAlarm			-1
#define apptMaxEndTime		0x1737	// 11:55 pm, hours in high byte, minutes in low byte

#define apptMaxDisplayableAlarms 10

#define datebookVersionNum						3
#define datebookPrefsVersionNum				4
#define datebookPrefID							0x00
#define datebookUnsavedPrefsVersionNum		1
#define datebookUnsavedPrefID					0x00
#define datebookDBName							"DatebookDB"

// Default setting for the Details Dialog.
#define defaultAlarmAdvance			5
#define defaultAdvanceUnit				aauMinutes


// Default databook app preference values
#define defaultDayStartHour					(8)
#define defaultDayEndHour						(18)
#define defaultAlarmPresetAdvance			(apptNoAlarm)
#define defaultAlarmPresetUnit				(aauMinutes)
#define defaultSaveBackup						(true)
#define defaultShowTimeBars					(true)
#define defaultCompressDayView				(true)
#define defaultShowTimedAppts					(true)
#define defaultShowUntimedAppts				(false)
#define defaultShowDailyRepeatingAppts		(false)
#define defaultAlarmSoundRepeatCount		(3)
#define defaultAlarmSoundRepeatInterval	(300)
#define defaultAlarmSoundUniqueRecID		(0)
#define defaultAlarmSnooze						(300)
#define defaultRecentForm						(DayView)

/************************************************************
 *
 * Appointment Database structures.
 *
 *************************************************************/

// The format of a packed appointment record is as follows:
//
//	ApptDateTimeType 		when;
//	ApptDBRecordFlags		flags;		// A flag set for each datum present
//	AlarmInfoType			alarm;		// optional
//	RepeatInfoType			repeat;		// optional
//	ExceptionsListType	exceptions;		// optional
//	char []					note;		// null terminated, optional
//	char []					description;	// null terminated
//
// All optional data may or may not appear.



// Alarm advance - the period of time before the appointment that the
// alarm should sound.
//
typedef enum alarmTypes {aauMinutes, aauHours, aauDays} AlarmUnitType;

typedef struct {
    Int8				advance;		// Alarm advance (-1 = no alarm)
    AlarmUnitType		advanceUnit;	// minutes, hours, days
} AlarmInfoType;


// The following enum is used to specify the frequency of
// repeating appointments.
//
enum repeatTypes {
    repeatNone,
    repeatDaily,
    repeatWeekly,
    repeatMonthlyByDay,
    repeatMonthlyByDate,
    repeatYearly
};
typedef enum repeatTypes RepeatType;


// This structure contains information about repeat appointments.  The
// repeatOn member is only used by weelky and monthly-by-day repeating
// appointments.  For weekly the byte is a bit field that contains the
// days of the week the appointments occurs on (bit: 0-sun, 1-mon,
// 2-tue, etc.).  For monthly-by-day the byte contains the day the
// appointments occurs, (ex: the 3rd friday), the byte is of type
// DayOfMonthType.
//
typedef struct {
    RepeatType		repeatType;			// daily, weekly, monthlyByDay, etc.
    DateType		repeatEndDate;	    // minus one if forever
    unsigned char	repeatFrequency;	// i.e. every 2 days if repeatType daily
    unsigned char	repeatOn;			// monthlyByDay and repeatWeekly only
    unsigned char  repeatStartOfWeek;   // repeatWeekly only
} RepeatInfoType;

typedef RepeatInfoType * RepeatInfoPtr;



// This structure contains information about the date and time of an
// appointment.
//
typedef struct {
    TimeType			startTime;		// Time the appointment starts
    TimeType			endTime;		// Time the appointment ends
    DateType			date;			// date of appointment
} ApptDateTimeType;



// This is the structure for a repeating appointment's exceptions list.  The
// exceptions list is a variable length list of dates that the repeating appointment
// should not appear on.  The list is in chronological order.
//
typedef struct {
    UInt16    	numExceptions;
    DateType	exception;
} ExceptionsListType;

typedef ExceptionsListType * ExceptionsListPtr;



// This structure describes what information is present in an
// AppointmentPackedDBRecordType
typedef struct {
unsigned when			:1;		// set if when info changed (ApptChangeRecord)
unsigned alarm			:1;		// set if record contains alarm info
unsigned repeat		:1;			// set if record contains repeat info
unsigned note			:1;		// set if record contains a note
unsigned exceptions	:1;			// set if record contains exceptions list
unsigned description	:1;
} ApptDBRecordFlags;


// ApptDBRecordType
//
// This is the record used by the application.  All pointers are either NULL
// or point to data within the PackedDB record.  All strings are null
// character terminated.

typedef struct {
    ApptDateTimeType *	when;
    AlarmInfoType *		alarm;
    RepeatInfoType *		repeat;
    ExceptionsListType *	exceptions;
    Char *					description;
    Char *					note;
} ApptDBRecordType;

// ApptGetAppointments returns an array of the following structures.
typedef struct {
    TimeType		startTime;
    TimeType		endTime;
    UInt16			recordNum;
} ApptInfoType;
typedef ApptInfoType * ApptInfoPtr;


typedef struct {
	UInt16		recordNum;
	UInt32		alarmTime;
} DatebookAlarmType;

typedef struct {
	UInt16 		recordNum;
	UInt32		alarmTime;
} PendingAlarmType;		// Moved from DateAlarm.c

typedef struct {
	UInt16					pendingCount;
	PendingAlarmType	pendingAlarm [apptMaxDisplayableAlarms];
	UInt16					dismissedCount;
	UInt32					dismissedAlarm [apptMaxDisplayableAlarms];
	UInt32					snoozeAnchorUniqueID;
	UInt32					snoozeAnchorTime;
	UInt16					snoozeDuration;
} PendingAlarmQueueType;

// This is the structure of the data that's saved to the state file.
typedef struct {
	UInt16					dayStartHour;
	UInt16					dayEndHour;
	AlarmInfoType		alarmPreset;
	FontID				v20NoteFont;		// Changed for 2.0 compatibility (BGT)
	Boolean				saveBackup;
	Boolean				showTimeBars;
	Boolean				compressDayView;
	Boolean				showTimedAppts;
	Boolean				showUntimedAppts;
	Boolean				showDailyRepeatingAppts;
	UInt8					reserved;
	
	// Version 3 preferences
	UInt16 					alarmSoundRepeatCount;
	UInt16 					alarmSoundRepeatInterval;
	UInt32					alarmSoundUniqueRecID;
	FontID				apptDescFont;
	FontID				noteFont;		// Changed for 2.0 compatibility (BGT)
	
	// Version 4 preferences
	UInt16					alarmSnooze;
} DatebookPreferenceType;

typedef ApptDBRecordType*   ApptDBRecordPtr;

/***********************************************************************
 *  
 *  Internal Structutes
 *  
 ***********************************************************************/
    
// The following structure doesn't really exist.  The first field
// varies depending on the data present.  However, it is convient
// (and less error prone) to use when accessing the other information.
typedef struct {
    ApptDateTimeType    when;
    ApptDBRecordFlags   flags;  // A flag set for each  datum present
    char                firstField;
} ApptPackedDBRecordType;

typedef ApptPackedDBRecordType * ApptPackedDBRecordPtr;
 
Int16 DateCompare(DateType d1, DateType d2) SECT2;
RepeatInfoPtr ApptGetRepeatInfo(ApptPackedDBRecordPtr src) SECT2;
void ApptUnpack(ApptPackedDBRecordPtr src, ApptDBRecordPtr dest) SECT2;
Boolean ApptFindFirst (DmOpenRef dbP, DateType date, UInt16 * indexP) SECT2;
Err ApptGetRecord (DmOpenRef dbP, UInt16 index, ApptDBRecordPtr r,
                   MemHandle * handleP) SECT2;
Err ApptChangeRecord(DmOpenRef dbP, UInt16 *index, ApptDBRecordPtr r,
                     ApptDBRecordFlags changedFields) SECT2; 
Boolean ApptRepeatsOnDate (ApptDBRecordPtr apptRec, DateType date) SECT2;
MemHandle ApptGetAppointments (DmOpenRef dbP, DateType date, UInt16 * countP) SECT2;
Err ApptNewRecord(DmOpenRef dbP, ApptDBRecordPtr r, UInt16 *index) SECT2; 
void RescheduleAlarms (DmOpenRef dbP) SECT2;

Boolean NextRepeat (ApptDBRecordPtr apptRec, DatePtr dateP) SECT2;
Boolean ApptNextRepeat (ApptDBRecordPtr apptRec, DatePtr dateP)  SECT2;
UInt32 ApptGetAlarmTime (ApptDBRecordPtr apptRec, UInt32 currentTime)  SECT2;
UInt32 AlarmGetTrigger (UInt32* refP)  SECT2;
void AlarmSetTrigger (UInt32 alarmTime, UInt32 ref)  SECT2;
UInt16 GetDismissedAlarmCount (PendingAlarmQueueType*	inAlarmInternalsP )  SECT2;
UInt32 * GetDismissedAlarmList (PendingAlarmQueueType*	inAlarmInternalsP )  SECT2;
UInt32 ApptAlarmMunger (
	DmOpenRef					inDbR,
	UInt32						inAlarmStart,
	UInt32						inAlarmStop,
	PendingAlarmQueueType *	    inAlarmInternalsP,
	UInt16*					    ioCountP,
	DatebookAlarmType *		    outAlarmListP,
	Boolean*					outAudibleP )  SECT2;
UInt32 ApptGetTimeOfNextAlarm (
	DmOpenRef					inDbR,			    // the open database
	UInt32						inAlarmStart ) SECT2;	// find first alarm on or after this time (in seconds)
void ReleaseAlarmInternals (
	PendingAlarmQueueType *	inAlarmInternalsP,
	Boolean						inModified ) SECT2;
void ReserveAlarmInternals (
	PendingAlarmQueueType*		outAlarmInternalsP ) SECT2;

#endif
